﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.Instances.Regression {
  public class KeijzerFunctionEight : ArtificialRegressionDataDescriptor {

    public override string Name { get { return "Keijzer 8 f(x) = log(x)"; } }
    public override string Description {
      get {
        return "Paper: Improving Symbolic Regression with Interval Arithmetic and Linear Scaling" + Environment.NewLine
          + "Authors: Maarten Keijzer" + Environment.NewLine
          + "Function: f(x) = log(x)" + Environment.NewLine
          + "range(train): x = [0:1:100]" + Environment.NewLine
          + "range(test): x = [0:0.1:100]" + Environment.NewLine
          + "Function Set: x + y, x * y, 1/x, -x, sqrt(x)" + Environment.NewLine + Environment.NewLine
          + "Note: The problem starts with 1 to avoid log(0), which is minus infinity!";
      }
    }
    protected override string TargetVariable { get { return "F"; } }
    protected override IEnumerable<string> InputVariables { get { return new List<string>() { "X", "F" }; } }
    protected override IEnumerable<string> AllowedInputVariables { get { return new List<string>() { "X" }; } }
    protected override int TrainingPartitionStart { get { return 0; } }
    protected override int TrainingPartitionEnd { get { return 100; } }
    protected override int TestPartitionStart { get { return 100; } }
    protected override int TestPartitionEnd { get { return 1091; } }

    protected override double[,] GenerateValues() {
      List<List<double>> data = new List<List<double>>();
      data.Add(ValueGenerator.GenerateSteps(1, 100, 1));
      data[0].AddRange(ValueGenerator.GenerateSteps(1, 100, 0.1));

      double x;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x = data[0][i];
        results.Add(Math.Log(x));
      }
      data.Add(results);

      return ValueGenerator.Transformation(data);
    }
  }
}
